
;Analog to Digital Conversion
;This code will read one analog input at pin AN4.
;The code will then output the corresponding 10bit decimal value to RB0-RB7, RA6, RA7. (RA7 is MSB)

;Call proper header, include and source files to initialize PIC

 	LIST  p=16F88    ; List directive to define processor
 	#INCLUDE <p16F88.inc>    ; Processor specific variable definitions

;Configure PIC optional properties

 __CONFIG    _CONFIG1, _LVP_OFF & _MCLR_OFF & _BODEN_OFF & _INTRC_IO & _PWRTE_OFF & _WDT_OFF 
			;Config directive is used to embed configuration word within .asm file.
			;The labels following the directive are located in the respective .inc file.
			;See datasheet for information on configuration word settings.

 errorlevel -302	;suppresses irrelavent error level

;Define Constant Variables and Registers 
	CBLOCK 0x20		
	DELAY 				 
	;ADC_X_STORAGE_LOW 	
	ADC_X_STORAGE_HIGH
	ENDC

	org 0x00		;sets origin of code 

;Initialize Inputs/Outputs, and Registers on the PIC16F88.
;Initialize TRISA and TRISB registers. 
 	bsf 	STATUS, RP0		;Sets bit RP0 in address STATUS to 1 (switches from bank 0 to 1).
 	movlw 	B'00000000'		;Assigns RB0-RB7 as outputs. 
 	movwf 	TRISB			;Writes value to TRISB register.
 	movlw	B'00010000'		;Assigns AN7 as input, all else as outputs. 
	movwf	TRISA			;Writes value to TRISA register. 
	movlw	b'00010000'		;Analog input code; Bit 7=unimplemented (0), Bit 6-0 = AN<6:0> where 1 = Analog I/O and 0 = Digital I/O
 	movwf	ANSEL			;Load it into ANSEL (this configures the function of the port pins)
	movlw	0x70
	movwf	OSCCON			;Loads internal 8MHz clock
    bcf 	STATUS, RP0		;Shifts back to Bank 0.

;Start of Main Program

START_MAIN_PROGRAM
	goto 	ADC_GET_X

;Initalization of 'Main Program' section. 

;Get Analog Input X from AN0
ADC_GET_X
	;clrf	ADC_X_STORAGE_LOW	;Clearing the working register
	clrf	ADC_X_STORAGE_HIGH	;Clearing the working register
	;bcf		PORTA, 0
	;bcf		STATUS, C
	bsf		STATUS,RP0			;Moves back to Bank1 to write to ADCON1	
	movlw	b'01000000'			;Bit 7 = AD Format, 1 = Right Justified, 0 = Left justified; Bit 6 = ADSC2 (Clock /2 select, 0 = off)
								;Bit 5-4 = VCFG<1:0> = A/D Voltage Reference Config bits, 00 = Vdd as +Vref, and Vss as -Vref
								;Bit 3-0 = unimplemented, read as 0
	movwf 	ADCON1				;Move new value into ADCON1 (This is an A/D Control Register @ 9Fh)	
	bcf		STATUS,RP0			;Move to Bank0 to write to ADCON0
	movlw	b'11100001'			;Bit 7-6 (ADCS<1:0>) sets the clock to be derived from the internal A/D module RC ocsilliator
								;Bit 5-3 (CHS<2:0>) Analog select bits. (000 = AN0...111 = AN7)
								;Bit 2 = GO/DONE bit which is the AD/Conversion Status. (1 = In Progress, 0 = off)
								;if ADON=1 and GO/DONE=1, conversion in progress, 0 = not in progress. Bit 1 = unimplemented.
								;Bit 0 = AD on bit; 1 = AD is on, 0 = AD is off and consumes no operating current.
	movwf	ADCON0				;Move new value into ADCON0 (This is an A/D Control Register @ 1Fh)
	bcf		STATUS,RP0			;Moves back to Bank0 for data manipulation.

    
	movlw	b'11111111'			;Sets binary value into DELAY for desired delay time.
	movwf	DELAY				;Executes delay loop.
                  
WAIT_X	
	decfsz	DELAY, 1			;Delay for 256 instruction cycles
	goto	WAIT_X				;Wait for A/D acquisition time  
	bcf		PIR1,ADIF           ;Clear A/D interruption flag bit                 
	goto 	WAIT1_X				;Goes to AD acquistion time loop

WAIT1_X
    btfsc   ADCON0,GO_DONE   	;Tests GO bit to see if A/D conversion is complete, 1 = In Progress; 0 = Not In Progress
    goto    WAIT1_X				;Wait for conversion to complete

    movf	ADRESH,W			;get conversion high byte
	movwf	ADC_X_STORAGE_HIGH	;store it in AD_X_STORAGE_HIGH
	;bcf		STATUS,RP1			;clear bank 2/3
	;bsf		STATUS,RP0			;Switch to bank1
	;movf	ADRESL,W			;get conversion low byte
	;movwf	ADC_X_STORAGE_LOW	;store it in AD_X_STORAGE_LOW

	;bcf		STATUS,RP0			;Switch to bank0
	bcf 	ADCON0,GO_DONE		;Ensures A/D Cnversion is not in progress.
	movlw	ADC_X_STORAGE_HIGH	;Move the results to the output pins
	movwf	PORTB	
	;movlw	ADC_X_STORAGE_LOW	;Move the results to the output pins
	;movwf	PORTB

	movlw	b'11111111'
	movwf	DELAY
WAIT_OUTPUT
	decfsz	DELAY, 1
	goto	WAIT_OUTPUT	

	goto	START_MAIN_PROGRAM
	
	end